Option Explicit

'PinPairLengthToExcel_101  -PADS PowerPCB VB script 
'Created 30.11.09 Ron Olinyk, Mindlink Technologies
'*******
'Generates a report in Excel Format of routed length of all connections (pin-pairs) in a net(s).
'Select net(s) in design, then run script
'If no nets selected, all nets in design are reported.


'**** USER Selection ****
'Assign max connection count per net
'User can modify to eliminate power nets ie. set value to 5
Const max_conn As Integer = 1000

'Assign units decimal places
Const dp_mils As String = "0.0"   'mils
Const dp_mm As String = "0.00"    'mm
Const dp_in As String = "0.000"   'inches

'Array of column names. You can modify it to rename columns
'Const Columns = Array("Net Name", "Pin Pair", "Length")   'display by pin-pair
Const Columns = Array("Net Name", "Pin1", "Pin2", "Length")  'display by pin

'************************

Dim fname As String
Dim len_fmt As String

Sub Main
    Dim net_cnt As Integer
    Dim conn_cnt As Integer
    Dim cnt As Integer
    Dim tempFile As String
    Dim i As Integer
    Dim nobj As Object
    Dim aNet As Object
    
	fname = ActiveDocument
	Unit_fncn()
	If fname = "" Then
		fname = "Untitled"
	End If
	tempFile = DefaultFilePath & "\temp.txt"
	Open tempFile For Output As #1

	StatusBarText = "Generating report..."
	'Output table header
	For i = 0 to UBound(Columns)
		Build_temp Columns(i)
	Next
	Print #1
	'Output table rows
	Set nobj = ActiveDocument.GetObjects(ppcbObjectTypeNet,,True)
	If nobj.Count < 1 Then
	  For Each aNet In ActiveDocument.Nets
		conn_cnt = anet.Connections.Count
		If conn_cnt < max_conn Then
		  For cnt = 1 To conn_cnt
		    Build_temp aNet.Name
		    Build_temp aNet.Connections(cnt).Pins(1)    'display by pin
		    Build_temp aNet.Connections(cnt).Pins(2)    'display by pin
		    'Build_temp aNet.Connections(cnt).Name      'display by pin-pair
		    Build_temp Format(aNet.Connections(cnt).Length, len_fmt)
		    Print #1
		  Next cnt
		End If
	  Next aNet
	Else
	  For Each aNet In nobj
	    conn_cnt = anet.Connections.Count
	    If conn_cnt < max_conn Then
		  For cnt = 1 To conn_cnt
		    Build_temp aNet.Name
		    Build_temp aNet.Connections(cnt).Pins(1)   'display by pin
		    Build_temp aNet.Connections(cnt).Pins(2)   'display by pin
		    'Build_temp aNet.Connections(cnt).Name     'display by pin-pair
		    Build_temp Format(aNet.Connections(cnt).Length, len_fmt)
		    Print #1
		  Next cnt
		End If  
	  Next aNet
	End If

	StatusBarText = ""
	Close #1
	ExportToExcel
End Sub

Sub ExportToExcel

Dim xl As Object

    'get temp file data into Clipboard
    FillClipboard
    
    'make OLE connection to Excel
	On Error Resume Next
	Set xl =  GetObject(,"Excel.Application")
	On Error GoTo ExcelError	' Enable error trapping.
	If xl Is Nothing Then
		Set xl =  CreateObject("Excel.Application")
	End If
	
	'transfer data to Excel
	xl.Visible = True
	xl.Workbooks.Add
	xl.ActiveSheet.Paste    'paste string data from clipboard into Excel
	'make it pretty
	xl.Columns("B:D").NumberFormat = len_fmt
	xl.Range("A1:D1").Font.Bold = True
	xl.Range("A1:D1").NumberFormat = "@"
	xl.ActiveSheet.UsedRange.Columns.AutoFit
	
	'stick Report Header into top row
	xl.Rows(1).Insert
	xl.Rows(1).Cells(1) = Space(1) & "Net Report for " & fname & " on " & Now & "  Units: " & Unit_fncn()
	xl.Rows(2).Insert
	xl.Rows(1).Font.bold = True
	xl.Range("A1").Select
	On Error GoTo 0 ' Disable error trapping. 
	Exit Sub    

ExcelError:
	MsgBox Err.Description, vbExclamation, "Error Running Excel"
	On Error GoTo 0 ' Disable error trapping.    
	Exit Sub
End Sub

'Concatenate data strings to temp file
Sub Build_temp (txt As String)
	Print #1, txt; vbTab;
End Sub

Private Sub FillClipboard

Dim tempFile As String
Dim L As Long
Dim AllData As String

	StatusBarText = "Export Data To Clipboard..."
	' Load whole file to string variable    
	tempFile = DefaultFilePath & "\temp.txt"
	Open tempFile  For Input As #1
	L = LOF(1)
	AllData = Input$(L,1)
	Close #1
	'Copy data to clipboard
	Clipboard AllData 
	Kill tempFile
	StatusBarText = ""
	
End Sub

'************************
' Get Current Design Units
Private Function Unit_fncn() As String

Dim currUnit As Integer
Dim currunitName As String

	currUnit = ActiveDocument.unit
    If currUnit = 2 Then
      currunitName = "mils"
      len_fmt = dp_mils
    ElseIf currUnit = 3 Then
      currunitName = "inches"
      len_fmt = dp_in
    ElseIf currUnit = 4 Then
      currunitName = "mm"
      len_fmt = dp_mm
    End If	
    Unit_fncn = currunitName
End Function
